package test;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Optional;
import java.util.stream.Stream;

import test.anotacije.*;

public class Test {

	private Tacka t1, t2, t3;
	
	private static Constructor<Tacka> konstruktor;
	private static Method dohvX, rastojanje;
	
	private static int osvojeno = 0, max = 0;
	
	static {
		try {
			konstruktor = Tacka.class.getConstructor(Double.TYPE, Double.TYPE);
		} catch (NoSuchMethodException | SecurityException e) {
			konstruktor = null;
		}
		try {
			dohvX = Stream.of(Tacka.class.getMethods())
					.filter(m->m.getReturnType().equals(Double.TYPE))
					.filter(m->m.getParameterCount() == 0)
					.findFirst()
					.orElse(null);
			rastojanje = Stream.of(Tacka.class.getMethods())
					.filter(m->m.getReturnType().equals(Double.TYPE))
					.filter(m->m.getParameterCount() == 1)
					.filter(m->m.getParameterTypes()[0].equals(Tacka.class))
					.findFirst()
					.orElse(null);
		} catch (SecurityException e) {}
	}

	@Pre
	public void preTesta() {
		System.out.println("Pocetak izvrsavanja testa...");
		try {
			t1 = konstruktor.newInstance(0.0, 0.0);
			t2 = konstruktor.newInstance(3.0, 0.0);
			t3 = konstruktor.newInstance(0.0, 4.0);
		} catch (InstantiationException | IllegalAccessException | IllegalArgumentException
				| InvocationTargetException e) {
			System.err.println(e);
			t1 = t2 = t3 = null;
		}
	}

	@Posle
	public void posleTesta() {
		System.out.println("Izvrsavanje testa zavrseno...");
	}

	@MojTest(naziv = "Test kreiranje", poeni = 1)
	public boolean test1() {
		return t1 != null;
	}

	@MojTest(naziv = "Test dohvatanje x", poeni = 2)
	public boolean test2() {
		try {
			return (double)dohvX.invoke(t1) == 0 && (double)dohvX.invoke(t2) == 3;
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			return false;
		}
	}

	@MojTest(naziv = "Test rastojanje 1", poeni = 3)
	public boolean test3() {
		try {
			return (double)rastojanje.invoke(t1, t3) == 4;
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			return false;
		}
	}
	
	@Preskoci
	@MojTest(naziv = "Test rastojanje 2", poeni = 3)
	public boolean test4() {
		try {
			return (double)rastojanje.invoke(t1, t3) == 4;
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			return false;
		}
	}

	public static void main(String[] args) {
		Test test = new Test();
		Method[] niz = Test.class.getDeclaredMethods();
		Optional<Method> pre = Arrays.stream(niz)
				.filter(m -> m.isAnnotationPresent(Pre.class))
				.findFirst();
		Optional<Method> posle = Arrays.stream(niz)
				.filter(m -> m.isAnnotationPresent(Posle.class))
				.findFirst();

		Method[] testovi = Arrays.stream(niz)
			.filter(m -> !m.isAnnotationPresent(Preskoci.class))
			.filter(m->m.isAnnotationPresent(MojTest.class))
			.sorted((m1,m2)->m1.getName().compareTo(m2.getName()))
			.toArray(Method[]::new);
		
		for(Method m:testovi) {
			try {
				MojTest anotacija = (MojTest)m.getAnnotation(MojTest.class);
				
				int poena = anotacija.poeni();
				max += poena;
				
				String naziv = anotacija.naziv();
				System.out.println("Test: "+naziv);
				
				if (pre.isPresent())
					pre.get().invoke(test);
				
				boolean uspesno = (boolean) m.invoke(test);
				if(uspesno) {
					osvojeno += poena;
					System.out.println("Test je uspesno izvrsen...");
				}else {
					System.out.println("Greska prilikom izvrsavanja testa...");
				}
				
				if (posle.isPresent())
					posle.get().invoke(test);
			} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
				System.out.println("Greska prilikom izvrsavanja testa...");
				System.err.println(e);
			}
			System.out.println();
		}
		
		System.out.println(String.format("%d / %d osvojenih poena", osvojeno, max));
	}
}
